// Define the predefined-queries module, specifying its dependencies. This module is responsible
// for invoking the queries that are defined on the metrics server.
define([    'jquery'],
	function($      ){
		var construct_error_message = function(jqXHR, textStatus, errorThrown) {
			var message = jqXHR.responseText;
			if(!message) {
				if(jqXHR.status === 0 && jqXHR.statusText == "error") {
					message = "Error: Page could not connect to metrics server";
				} else if(textStatus == "parsererror") {
					message = "Error: Could not parse the data that was returned by the metrics server: " + errorThrown.message;
				} else if(errorThrown) {
					message = errorThrown.message;
				} else {
					message = "Error"; 
				}
			}
			if(jqXHR.status != 200 && jqXHR.statusText != "OK") {
				message += " (" + jqXHR.status + " " + jqXHR.statusText + ")";
			}
			return message;
		}
	
		var run_query_for_uri_suffix = function(uri_suffix, callbacks) {
			var url = "/metric/predefined-queries/" + uri_suffix;
			if(window.metricsServerUrl) {
				url = window.metricsServerUrl + url;
			}
			
			/*
			 * Ajax calls are in native JS, not GWT as:
			 * 1. jQuery or XmlHttpRequest are far more standard ways of doing them than GWT's RequestBuilder.
			 * 2. The slowest part of loading this is the server-side query, so we may want to do that (via
			 * XmlHttpRequest) before loading any other JS at all, including GWT.
			 * 
			 * Everything else in native JS should be kept as simple as possible (to avoid the need for JS unit
			 * tests) - also we're re-using the translation & accessibility work already done for GWT.  
			 */
			$.ajax({
				type: "GET",
				url: url,
				dataType: 'json',
				success: function (data, textStatus, jqXHR) {
					setTimeout(function(){
						callbacks.success(data.queries);
					}, 0);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					console.error(arguments);
					var message = construct_error_message(jqXHR, textStatus, errorThrown);
					callbacks.error(message);
				}
			});
		}

		
		// Create the public object for this module
		var predefined_queries = {};
			
		predefined_queries.run_query_for_domain = function(query_name, query_params, callbacks) {
			// Use the metrics server query name and a URI query string in the URI to call
			let ret = []
			for ( let param in query_params) {
				if (query_params[param]) {
					ret.push(encodeURIComponent(param) + "=" + encodeURIComponent(query_params[param]));
				}
			}

			let uri_suffix = query_name;
			if (ret.length > 0) {
				uri_suffix += "?" + ret.join("&");
			}
			
			// Wrap the callbacks to include the query parameters in the success value
			var wrapped_callbacks = {
				success: function(kairosResults) {
					callbacks.success({
						parameters: {
							period: query_name,
							domain: query_params.domain,
							start: query_params.start,
							end: query_params.end,
							unit: query_params.unit
						},
						kairosResults: kairosResults
					});
				},
				error: callbacks.error
			};
			
			run_query_for_uri_suffix(uri_suffix, wrapped_callbacks);
		};
		
		predefined_queries.run = function(query_name, callbacks) {
			// Query name is the suffix (last path entry) for the URI
			predefined_queries.run_query_for_domain(query_name, null, callbacks);
		};
			
		predefined_queries.get_domains = function(callbacks) {
			var url = "/metric/predefined-queries/domains";
			if(window.metricsServerUrl) {
				url = window.metricsServerUrl + url;
			}
			$.ajax({
				type: "GET",
				url: url,
				dataType: 'json',
				success: function (data, textStatus, jqXHR) {
					callbacks.success(data);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					console.error(arguments);
					var message = construct_error_message(jqXHR, textStatus, errorThrown);
					callbacks.error(message);
				}
			});
		};
		
		// Return the object that will be passed to other modules if they state
		// a dependency on 'rtcp/predefined-queries' (through calls to
		// require.js)
		return predefined_queries;
	}
);